import math

import rospy
from clever import srv
from std_srvs.srv import Trigger

from modules_for_circle import circle_perimeter

rospy.init_node('fly')

# get_telemetry = rospy.ServiceProxy('get_telemetry', srv.GetTelemetry)
navigate = rospy.ServiceProxy('navigate', srv.Navigate)
set_position = rospy.ServiceProxy('set_position', srv.SetPosition)
set_attitude = rospy.ServiceProxy('set_attitude', srv.SetAttitude)
land = rospy.ServiceProxy('land', Trigger)


class CyrcleFly():
    def __init__(self, circle_centre_x, circle_centre_y, radius, work_time=3.0, start_ange=0, turn_side=1, debug=0):
        self.phi = start_ange
        self.centre_x = circle_centre_x
        self.centre_y = circle_centre_y
        self.radius = radius
        self.turn_side = turn_side
        self.angle_diff = 5 * self.turn_side

        self.heigth = 2
        self.debug = debug

        circle_perim = circle_perimeter(self.radius)
        self.work_time = work_time

    @staticmethod
    def first_up(first_x, firxt_y, start_angle):
        navigate(x=0, y=0, z=1.5, speed=2, auto_arm=True, frame_id='body')
        rospy.sleep(3)

        navigate(x=first_x, y=firxt_y, z=2, yaw=math.radians(start_angle), frame_id='aruco_map')
        rospy.sleep(4)

    def fly_next_step(self, step_x, step_y):
        set_position(x=self.centre_x + step_x,
                     y=self.centre_y + step_y,
                     z=2,
                     yaw=math.radians(self.phi + 90 * self.turn_side),
                     frame_id='aruco_map')

    def calc_next_step(self):
        delta_x = self.radius * math.cos(math.radians(self.phi))
        delta_y = self.radius * math.sin(math.radians(self.phi))

        if self.debug == 1:
            print("Fly circle in global map to: x-->{}, y-->{}".format(self.centre_x + delta_x, self.centre_y + delta_y))

        self.fly_next_step(delta_x, delta_y)

        self.phi += self.angle_diff

    def main_f(self, up_x, up_y):
        self.first_up(up_x, up_y)

        try:
            while True:
                self.calc_next_step()
                rospy.sleep(self.sleep_time)

        except KeyboardInterrupt:
            land()
            rospy.sleep(6)

    def fly_at_an_angle(self, initial_angle, rotate_angle):
        start_angle = rotate_angle
        self.phi = initial_angle

        self.sleep_time = (self.work_time / (rotate_angle / math.fabs(self.angle_diff))) * 0.95

        while start_angle > 0:
            self.calc_next_step()
            rospy.sleep(self.sleep_time)

            start_angle -= math.fabs(self.angle_diff)

    def set_phi(self, phi):
        self.phi = math.radians(phi)

    def change_rotation_side(self, rotation_side):
        if rotation_side == -1 or rotation_side == "clock":
            self.angle_diff = math.fabs(self.angle_diff) * -1
        elif rotation_side == 1 or rotation_side == "counterclock":
            self.angle_diff = math.fabs(self.angle_diff)


def lets_try_fly_insinity():
    fly_left = CyrcleFly(3, 3, 1)
    fly_right = CyrcleFly(5, 3, 1, turn_side=-1)

    fly_left.first_up(3, 4)

    while True:
        fly_left.fly_at_an_angle(0, 360)
        fly_right.fly_at_an_angle(180, 360)


if __name__ == "__main__":
    counter = 0

    left_circle = CyrcleFly(0.2, 2, 1.4, turn_side=-1)
    right_circle = CyrcleFly(5.25, 2.1, 1.4)

    left_circle.first_up(0.95, 0.76, 230)

    while True:
        if counter >= 5:
            land()
            break

        left_circle.fly_at_an_angle(-70, 230)
        left_circle.set_phi(-70)
        navigate(x=4.2, y=0.9, z=2, yaw=math.radians(-35), speed=3, frame_id='aruco_map')
        rospy.sleep(2.2)

        right_circle.fly_at_an_angle(-120, 260)
        right_circle.set_phi(-125)
        navigate(x=1, y=0.82, z=2, yaw=math.radians(-150), speed=3, frame_id='aruco_map')
        rospy.sleep(2.2)
        counter += 1

